import React, { Component, ErrorInfo, ReactNode } from 'react';
import Link from 'next/link';
import { FaExclamationTriangle, FaHome, FaRedo } from 'react-icons/fa';
import * as Sentry from '@sentry/react';
import styles from '@/styles/ErrorBoundary.module.scss';

interface Props {
  children: ReactNode;
  fallback?: ReactNode;
  showDetail?: boolean;
  onError?: (error: Error, errorInfo: ErrorInfo) => void;
}

interface State {
  hasError: boolean;
  error: Error | null;
  errorInfo: ErrorInfo | null;
}

class ErrorBoundary extends Component<Props, State> {
  constructor(props: Props) {
    super(props);
    this.state = {
      hasError: false,
      error: null,
      errorInfo: null,
    };
  }

  static getDerivedStateFromError(error: Error): Partial<State> {
    return { hasError: true, error };
  }

  componentDidCatch(error: Error, errorInfo: ErrorInfo): void {
    this.setState({ errorInfo });
    
    // Log to console
    console.error("Error caught by ErrorBoundary:", error, errorInfo);
    
    // Send to Sentry if configured
    if (process.env.NEXT_PUBLIC_SENTRY_DSN) {
      Sentry.captureException(error);
    }
    
    // Call onError prop if provided
    if (this.props.onError) {
      this.props.onError(error, errorInfo);
    }
  }
  
  handleRetry = (): void => {
    this.setState({ hasError: false, error: null, errorInfo: null });
  }

  render(): ReactNode {
    const { hasError, error } = this.state;
    const { children, fallback, showDetail } = this.props;
    
    if (!hasError) {
      return children;
    }
    
    // Use custom fallback if provided
    if (fallback) {
      return fallback;
    }
    
    // Default error UI
    return (
      <div className={styles.errorContainer}>
        <div className={styles.errorContent}>
          <div className={styles.errorIcon}>
            <FaExclamationTriangle />
          </div>
          <h2>Đã có lỗi xảy ra</h2>
          <p>
            Rất tiếc, đã có lỗi xảy ra khi hiển thị nội dung này.
            {showDetail && error && (
              <details className={styles.errorDetails}>
                <summary>Xem chi tiết lỗi</summary>
                <p>{error.toString()}</p>
              </details>
            )}
          </p>
          <div className={styles.errorActions}>
            <button 
              onClick={this.handleRetry} 
              className={styles.retryButton}
              data-testid="error-retry-button"
            >
              <FaRedo /> Thử lại
            </button>
            <Link href="/" className={styles.homeButton}>
              <FaHome /> Trang chủ
            </Link>
          </div>
        </div>
      </div>
    );
  }
}

export default ErrorBoundary;